@@ -1,7 +1,6 @@
 module Puppet::Pops
 module Types
-  EMPTY_ARRAY = [].freeze
-
+  # @api private
   class TypePathElement
     attr_reader :key
 
@@ -22,30 +21,46 @@ module Types
     end
    end
 
+  # @api private
   class SubjectPathElement < TypePathElement
     def to_s
       key
     end
   end
 
+  # @api private
   class EntryValuePathElement < TypePathElement
     def to_s
       "entry '#{key}'"
     end
   end
 
+  # @api private
   class EntryKeyPathElement < TypePathElement
     def to_s
       "key of entry '#{key}'"
     end
   end
 
+  # @api private
   class ParameterPathElement < TypePathElement
     def to_s
       "parameter '#{key}'"
     end
   end
 
+  # @api private
+  class ReturnTypeElement < TypePathElement
+    def initialize(name = 'return')
+      super(name)
+    end
+
+    def to_s
+      key
+    end
+  end
+
+  # @api private
   class BlockPathElement < ParameterPathElement
     def initialize(name = 'block')
       super(name)
@@ -56,69 +71,29 @@ module Types
     end
   end
 
+  # @api private
   class ArrayPathElement < TypePathElement
     def to_s
       "index #{key}"
     end
   end
 
+  # @api private
   class VariantPathElement < TypePathElement
     def to_s
       "variant #{key}"
     end
   end
 
+  # @api private
   class SignaturePathElement < VariantPathElement
     def to_s
       "#{key+1}."
     end
   end
 
-  # Module to handle present/past tense.
-  #
-  # All method names prefixed with "it_" to avoid conflict with Mocha expectations. Adding a method
-  # named 'expects' just doesn't work.
-  #
-  module TenseVariants
-    def it_expects(tense)
-      case tense
-      when :present
-        'expects'
-      else
-        'expected'
-      end
-    end
-
-    def it_does_not_expect(tense)
-      case tense
-      when :present
-        'does not expect'
-      else
-        'did not expect'
-      end
-    end
-
-    def it_has_no(tense)
-      case tense
-      when :present
-        'has no'
-      else
-        'did not have a'
-      end
-    end
-
-    def it_references(tense)
-      case tense
-        when :present
-          'references'
-        else
-          'referenced'
-      end
-    end
-  end
-
+  # @api private
   class Mismatch
-    include TenseVariants
     attr_reader :path
 
     def initialize(path)
@@ -129,7 +104,7 @@ module Types
       @canonical_path ||= @path.reject { |e| e.is_a?(VariantPathElement) }
     end
 
-    def message(variant, position, tense = :present)
+    def message(variant, position)
       "#{variant}unknown mismatch#{position}"
     end
 
@@ -163,10 +138,10 @@ module Types
     end
 
     def to_s
-      format(:present)
+      format
     end
 
-    def format(tense)
+    def format
       p = @path
       variant = ''
       position = ''
@@ -178,11 +153,12 @@ module Types
         end
         position = " #{p.join(' ')}" unless p.empty?
       end
-      message(variant, position, tense)
+      message(variant, position)
     end
   end
 
   # @abstract
+  # @api private
   class KeyMismatch < Mismatch
     attr_reader :key
 
@@ -200,42 +176,49 @@ module Types
     end
   end
 
+  # @api private
   class MissingKey < KeyMismatch
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_expects(tense)} a value for key '#{key}'"
+    def message(variant, position)
+      "#{variant}#{position} expects a value for key '#{key}'"
     end
   end
 
+  # @api private
   class MissingParameter < KeyMismatch
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_expects(tense)} a value for parameter '#{key}'"
+    def message(variant, position)
+      "#{variant}#{position} expects a value for parameter '#{key}'"
     end
   end
 
+  # @api private
   class ExtraneousKey < KeyMismatch
-    def message(variant, position, tense = :present)
+    def message(variant, position)
       "#{variant}#{position} unrecognized key '#{@key}'"
     end
   end
 
+  # @api private
   class InvalidParameter < ExtraneousKey
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_has_no(tense)} parameter named '#{@key}'"
+    def message(variant, position)
+      "#{variant}#{position} has no parameter named '#{@key}'"
     end
   end
 
+  # @api private
   class UnexpectedBlock < Mismatch
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_does_not_expect(tense)} a block"
+    def message(variant, position)
+      "#{variant}#{position} does not expect a block"
     end
   end
 
+  # @api private
   class MissingRequiredBlock < Mismatch
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_expects(tense)} a block"
+    def message(variant, position)
+      "#{variant}#{position} expects a block"
     end
   end
 
+  # @api private
   class UnresolvedTypeReference < Mismatch
     attr_reader :unresolved
 
@@ -252,11 +235,12 @@ module Types
       @unresolved.hash
     end
 
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_references(tense)} an unresolved type '#{@unresolved}'"
+    def message(variant, position)
+      "#{variant}#{position} references an unresolved type '#{@unresolved}'"
     end
   end
 
+  # @api private
   class ExpectedActualMismatch < Mismatch
     attr_reader :expected, :actual
 
@@ -273,14 +257,9 @@ module Types
     def hash
       [canonical_path, expected, actual].hash
     end
-
-    def swap_expected(expected)
-      copy = self.clone
-      copy.instance_variable_set(:@expected, expected)
-      copy
-    end
   end
 
+  # @api private
   class TypeMismatch < ExpectedActualMismatch
     include LabelProvider
 
@@ -289,20 +268,28 @@ module Types
       self.class.new(path, [expected, o.expected].flatten.uniq, actual)
     end
 
-    def message(variant, position, tense = :present)
+    def message(variant, position)
       e = expected
       a = actual
       multi = false
+      if e.is_a?(POptionalType)
+        e = e.optional_type
+        optional = true
+      end
+
       if e.is_a?(PVariantType)
         e = e.types
+      end
+
+      if e.is_a?(Array)
         if report_detailed?(e, a)
           a = detailed_actual_to_s(e, a)
           e = e.map { |t| t.to_alias_expanded_s }
         else
-          sns = e.map { |t| t.simple_name }.uniq
-          e = e.map { |t| s = t.simple_name; sns.count {|x| x == s } == 1 ? s : t.to_s }.uniq
-          a = a.simple_name
+          e = e.map { |t| short_name(t) }.uniq
+          a = short_name(a)
         end
+        e.insert(0, 'Undef') if optional
         case e.size
         when 1
           e = e[0]
@@ -318,14 +305,18 @@ module Types
           a = detailed_actual_to_s(e, a)
           e = e.to_alias_expanded_s
         else
-          e = e.simple_name
-          a = a.simple_name
+          e = short_name(e)
+          a = short_name(a)
+        end
+        if optional
+          e = "Undef or #{e}"
+          multi = true
         end
       end
       if multi
-        "#{variant}#{position} #{it_expects(tense)} a value of type #{e}, got #{label(a)}"
+        "#{variant}#{position} expects a value of type #{e}, got #{label(a)}"
       else
-        "#{variant}#{position} #{it_expects(tense)} #{a_an(e)} value, got #{label(a)}"
+        "#{variant}#{position} expects #{a_an(e)} value, got #{label(a)}"
       end
     end
 
@@ -335,6 +326,16 @@ module Types
 
     private
 
+    def short_name(t)
+      # Ensure that Optional, NotUndef, Sensitive, and Type are reported with included
+      # type parameter.
+      if t.is_a?(PTypeWithContainedType) && !(t.type.nil? || t.type.class == PAnyType)
+        "#{t.name}[#{t.type.name}]"
+      else
+        t.name.nil? ? t.simple_name : t.name
+      end
+    end
+
     # Answers the question if `e` is a specialized type of `a`
     # @param e [PAnyType] the expected type
     # @param a [PAnyType] the actual type
@@ -402,7 +403,7 @@ module Types
       if e.is_a?(Array)
         e.map { |t| all_resolved(t) }
       else
-        e.is_a?(PTypeAliasType) ? e.resolved_type : e
+        e.is_a?(PTypeAliasType) ? all_resolved(e.resolved_type) : e
       end
     end
 
@@ -420,23 +421,30 @@ module Types
       if always_fully_detailed?(e, a)
         a.to_alias_expanded_s
       else
-        g = a.generalize
-        any_assignable?(e, g) ? a.to_alias_expanded_s : g.to_s
+        any_assignable?(e, a.generalize) ? a.to_alias_expanded_s : a.simple_name
       end
     end
   end
 
+  # @api private
   class PatternMismatch < TypeMismatch
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_expects(tense)} a match for #{expected.to_alias_expanded_s}, got #{actual_string}"
+    def message(variant, position)
+      e = expected
+      value_pfx = ''
+      if e.is_a?(POptionalType)
+        e = e.optional_type
+        value_pfx = 'an undef value or '
+      end
+      "#{variant}#{position} expects #{value_pfx}a match for #{e.to_alias_expanded_s}, got #{actual_string}"
     end
 
     def actual_string
       a = actual
-      a.is_a?(PStringType) && a.values.size == 1 ? "'#{a.values[0]}'" : a.simple_name
+      a.is_a?(PStringType) && !a.value.nil? ? "'#{a.value}'" : short_name(a)
     end
   end
 
+  # @api private
   class SizeMismatch < ExpectedActualMismatch
     def from
       @expected.from || 0
@@ -452,8 +460,8 @@ module Types
       self.class.new(path, range, @actual)
     end
 
-    def message(variant, position, tense = :present)
-      "#{variant}#{position} #{it_expects(tense)} size to be #{range_to_s(expected, '0')}, got #{range_to_s(actual, '0')}"
+    def message(variant, position)
+      "#{variant}#{position} expects size to be #{range_to_s(expected, '0')}, got #{range_to_s(actual, '0')}"
     end
 
     def range_to_s(range, zero_string)
@@ -471,22 +479,22 @@ module Types
     end
   end
 
+  # @api private
   class CountMismatch < SizeMismatch
     def initialize(path, expected, actual)
       super(path, expected, actual)
     end
 
-    def message(variant, position, tense = :present)
+    def message(variant, position)
       min = expected.from || 0
       max = expected.to || Float::INFINITY
       suffix = min == 1 && (max == 1 || max == Float::INFINITY) || min == 0 && max == 1 ? '' : 's'
-      "#{variant}#{position} #{it_expects(tense)} #{range_to_s(expected, 'no')} argument#{suffix}, got #{range_to_s(actual, 'none')}"
+      "#{variant}#{position} expects #{range_to_s(expected, 'no')} argument#{suffix}, got #{range_to_s(actual, 'none')}"
     end
   end
 
+  # @api private
   class TypeMismatchDescriber
-    include TenseVariants
-
     def self.validate_parameters(subject, params_struct, given_hash, missing_ok = false)
       singleton.validate_parameters(subject, params_struct, given_hash, missing_ok)
     end
@@ -503,6 +511,13 @@ module Types
       @singleton ||= new
     end
 
+    def tense_deprecated
+      #TRANSLATORS TypeMismatchDescriber is a class name and 'tense' is a method name and should not be translated
+      message = _("Passing a 'tense' argument to the TypeMismatchDescriber is deprecated and ignored.")
+      message += ' ' + _("Everything is now reported using present tense")
+      Puppet.warn_once('deprecations', 'typemismatch#tense', message)
+    end
+
     # Validates that all entries in the give_hash exists in the given param_struct, that their type conforms
     # with the corresponding param_struct element and that all required values are provided.
     #
@@ -510,16 +525,17 @@ module Types
     # @param params_struct [PStructType] Struct to use for validation
     # @param given_hash [Hash<String,Object>] the parameters to validate
     # @param missing_ok [Boolean] Do not generate errors on missing parameters
-    # @param tense [Symbol] the symbol :present or :past
+    # @param tense [Symbol] deprecated and ignored
     #
-    def validate_parameters(subject, params_struct, given_hash, missing_ok = false, tense = :present)
+    def validate_parameters(subject, params_struct, given_hash, missing_ok = false, tense = :ignored)
+      tense_deprecated unless tense == :ignored
       errors = describe_struct_signature(params_struct, given_hash, missing_ok).flatten
       case errors.size
       when 0
       when 1
-        raise Puppet::ParseError.new("#{subject}:#{errors[0].format(tense)}")
+        raise Puppet::ParseError.new("#{subject}:#{errors[0].format}")
       else
-        errors_str = errors.map { |error| error.format(tense) }.join("\n ")
+        errors_str = errors.map { |error| error.format }.join("\n ")
         raise Puppet::ParseError.new("#{subject}:\n #{errors_str}")
       end
     end
@@ -529,17 +545,18 @@ module Types
     # @param name [String] name of mismatch
     # @param expected [PAnyType] expected type
     # @param actual [PAnyType] actual type
-    # @param tense [Symbol] the symbol :present or :past
+    # @param tense [Symbol] deprecated and ignored
     #
-    def describe_mismatch(name, expected, actual, tense = :past)
+    def describe_mismatch(name, expected, actual, tense = :ignored)
+      tense_deprecated unless tense == :ignored
       errors = describe(expected, actual, [SubjectPathElement.new(name)])
       case errors.size
       when 0
         ''
       when 1
-        errors[0].format(tense).strip
+        errors[0].format.strip
       else
-        errors.map { |error| error.format(tense) }.join("\n ")
+        errors.map { |error| error.format }.join("\n ")
       end
     end
 
@@ -547,17 +564,18 @@ module Types
     # @param param_name [String] parameter name
     # @param param_type [PAnyType] parameter type
     # @param value [Object] value to be validated against the given type
-    # @param tense [Symbol] the symbol :present or :past
+    # @param tense [Symbol] deprecated and ignored
     #
-    def validate_default_parameter(subject, param_name, param_type, value, tense = :present)
+    def validate_default_parameter(subject, param_name, param_type, value, tense = :ignored)
+      tense_deprecated unless tense == :ignored
       unless param_type.instance?(value)
         errors = describe(param_type, TypeCalculator.singleton.infer_set(value).generalize, [ParameterPathElement.new(param_name)])
         case errors.size
         when 0
         when 1
-          raise Puppet::ParseError.new("#{subject}:#{errors[0].format(tense)}")
+          raise Puppet::ParseError.new("#{subject}:#{errors[0].format}")
         else
-          errors_str = errors.map { |error| error.format(tense) }.join("\n ")
+          errors_str = errors.map { |error| error.format }.join("\n ")
           raise Puppet::ParseError.new("#{subject}:\n #{errors_str}")
         end
       end
@@ -580,15 +598,16 @@ module Types
         value = param_hash[name]
         value_type = elem.value_type
         if param_hash.include?(name)
-          result << describe(value_type, TypeCalculator.singleton.infer_set(value).generalize, [ParameterPathElement.new(name)]) unless value_type.instance?(value)
+          result << describe(value_type, TypeCalculator.singleton.infer_set(value), [ParameterPathElement.new(name)]) unless value_type.instance?(value)
         else
-          result << MissingParameter.new(nil, name) unless elem.key_type.assignable?(PUndefType::DEFAULT) unless missing_ok
+          result << MissingParameter.new(nil, name) unless missing_ok || elem.key_type.is_a?(POptionalType)
         end
       end
       result
     end
 
-    def describe_signatures(closure, signatures, args_tuple, tense = :present)
+    def describe_signatures(closure, signatures, args_tuple, tense = :ignored)
+      tense_deprecated unless tense == :ignored
       error_arrays = []
       signatures.each_with_index do |signature, index|
         error_arrays << describe_signature_arguments(signature, args_tuple, [SignaturePathElement.new(index)])
@@ -614,17 +633,17 @@ module Types
       label = closure == 'lambda' ? 'block' : "'#{closure}'"
       errors = merge_descriptions(0, CountMismatch, error_arrays)
       if errors.size == 1
-        "#{label}#{errors[0].format(tense)}"
+        "#{label}#{errors[0].format}"
       else
         if signatures.size == 1
           sig = signatures[0]
-          result = ["#{label} #{it_expects(tense)} (#{signature_string(sig)})"]
-          result.concat(error_arrays[0].map { |e| "  rejected:#{e.chop_path(0).format(tense)}" })
+          result = ["#{label} expects (#{signature_string(sig)})"]
+          result.concat(error_arrays[0].map { |e| "  rejected:#{e.chop_path(0).format}" })
         else
-          result = ["#{label} #{it_expects(tense)} one of:"]
+          result = ["#{label} expects one of:"]
           signatures.each_with_index do |sg, index|
             result << "  (#{signature_string(sg)})"
-            result.concat(error_arrays[index].map { |e| "    rejected:#{e.chop_path(0).format(tense)}" })
+            result.concat(error_arrays[index].map { |e| "    rejected:#{e.chop_path(0).format}" })
           end
         end
         result.join("\n")
@@ -682,8 +701,10 @@ module Types
         arg_count.times do |index|
           adx = index >= etypes.size ? etypes.size - 1 : index
           etype = etypes[adx]
-          descriptions = describe(etype, atypes[index], path + [ParameterPathElement.new(enames[adx])])
-          return descriptions unless descriptions.empty?
+          unless etype.assignable?(atypes[index])
+            descriptions = describe(etype, atypes[index], path + [ParameterPathElement.new(enames[adx])])
+            return descriptions unless descriptions.empty?
+          end
         end
         EMPTY_ARRAY
       else
@@ -691,14 +712,23 @@ module Types
       end
     end
 
-    def describe_PVariantType(expected, actual, path)
+    def describe_PVariantType(expected, original, actual, path)
       variant_descriptions = []
-      expected.types.each_with_index do |vt, index|
+      types = expected.types
+      types = [PUndefType::DEFAULT] + types if original.is_a?(POptionalType)
+      types.each_with_index do |vt, index|
         d = describe(vt, actual, path + [VariantPathElement.new(index)])
         return EMPTY_ARRAY if d.empty?
         variant_descriptions << d
       end
-      merge_descriptions(path.length, SizeMismatch, variant_descriptions)
+      descriptions = merge_descriptions(path.length, SizeMismatch, variant_descriptions)
+      if original.is_a?(PTypeAliasType) && descriptions.size == 1
+        # All variants failed in this alias so we report it as a mismatch on the alias
+        # rather than reporting individual failures of the variants
+        [TypeMismatch.new(path, original, actual)]
+      else
+        descriptions
+      end
     end
 
     def merge_descriptions(varying_path_position, size_mismatch_class, variant_descriptions)
@@ -722,30 +752,24 @@ module Types
       descriptions.size == 1 ? [descriptions[0].chop_path(varying_path_position)] : descriptions
     end
 
-    def describe_POptionalType(expected, actual, path)
-      actual.is_a?(PUndefType) ? [] : describe(expected.optional_type, actual, path)
+    def describe_POptionalType(expected, original, actual, path)
+      return EMPTY_ARRAY if actual.is_a?(PUndefType)
+      internal_describe(expected.optional_type, original.is_a?(PTypeAliasType) ? original : expected, actual, path)
     end
 
-    def describe_PEnumType(expected, actual, path)
-      [PatternMismatch.new(path, expected, actual)]
+    def describe_PEnumType(expected, original, actual, path)
+      [PatternMismatch.new(path, original, actual)]
     end
 
-    def describe_PPatternType(expected, actual, path)
-      [PatternMismatch.new(path, expected, actual)]
+    def describe_PPatternType(expected, original, actual, path)
+      [PatternMismatch.new(path, original, actual)]
     end
 
-    def describe_PTypeAliasType(expected, actual, path)
-      resolved_type = expected.resolved_type.normalize
-      describe(resolved_type, actual, path).map do |description|
-        if description.is_a?(ExpectedActualMismatch) && description.expected.equal?(resolved_type)
-          description.swap_expected(expected)
-        else
-          description
-        end
-      end
+    def describe_PTypeAliasType(expected, original, actual, path)
+      internal_describe(expected.resolved_type.normalize, expected, actual, path)
     end
 
-    def describe_PArrayType(expected, actual, path)
+    def describe_PArrayType(expected, original, actual, path)
       descriptions = []
       element_type = expected.element_type || PAnyType::DEFAULT
       if actual.is_a?(PTupleType)
@@ -763,20 +787,20 @@ module Types
         expected_size = expected.size_type
         actual_size = actual.size_type || PCollectionType::DEFAULT_SIZE
         if expected_size.nil? || expected_size.assignable?(actual_size)
-          descriptions << TypeMismatch.new(path, expected, actual)
+          descriptions << TypeMismatch.new(path, original, PArrayType.new(actual.element_type))
         else
           descriptions << SizeMismatch.new(path, expected_size, actual_size)
         end
       else
-        descriptions << TypeMismatch.new(path, expected, actual)
+        descriptions << TypeMismatch.new(path, original, actual)
       end
       descriptions
     end
 
-    def describe_PHashType(expected, actual, path)
+    def describe_PHashType(expected, original, actual, path)
       descriptions = []
       key_type = expected.key_type || PAnyType::DEFAULT
-      value_type = expected.element_type || PAnyType::DEFAULT
+      value_type = expected.value_type || PAnyType::DEFAULT
       if actual.is_a?(PStructType)
         elements = actual.elements
         expected_size = expected.size_type || PCollectionType::DEFAULT_SIZE
@@ -793,17 +817,17 @@ module Types
         expected_size = expected.size_type
         actual_size = actual.size_type || PCollectionType::DEFAULT_SIZE
         if expected_size.nil? || expected_size.assignable?(actual_size)
-          descriptions << TypeMismatch.new(path, expected, actual)
+          descriptions << TypeMismatch.new(path, original, PHashType.new(actual.key_type, actual.value_type))
         else
           descriptions << SizeMismatch.new(path, expected_size, actual_size)
         end
       else
-        descriptions << TypeMismatch.new(path, expected, actual)
+        descriptions << TypeMismatch.new(path, original, actual)
       end
       descriptions
     end
 
-    def describe_PStructType(expected, actual, path)
+    def describe_PStructType(expected, original, actual, path)
       elements = expected.elements
       descriptions = []
       if actual.is_a?(PStructType)
@@ -823,26 +847,26 @@ module Types
         actual_size = actual.size_type || PCollectionType::DEFAULT_SIZE
         expected_size = PIntegerType.new(elements.count { |e| !e.key_type.assignable?(PUndefType::DEFAULT) }, elements.size)
         if expected_size.assignable?(actual_size)
-          descriptions << TypeMismatch.new(path, expected, actual)
+          descriptions << TypeMismatch.new(path, original, PHashType.new(actual.key_type, actual.value_type))
         else
           descriptions << SizeMismatch.new(path, expected_size, actual_size)
         end
       else
-        descriptions << TypeMismatch.new(path, expected, actual)
+        descriptions << TypeMismatch.new(path, original, actual)
       end
       descriptions
     end
 
-    def describe_PTupleType(expected, actual, path)
-      describe_tuple(expected, actual, path, SizeMismatch)
+    def describe_PTupleType(expected, original, actual, path)
+      describe_tuple(expected, original, actual, path, SizeMismatch)
     end
 
     def describe_argument_tuple(expected, actual, path)
-      describe_tuple(expected, actual, path, CountMismatch)
+      describe_tuple(expected, expected, actual, path, CountMismatch)
     end
 
-    def describe_tuple(expected, actual, path, size_mismatch_class)
-      return if expected == actual || expected.types.empty? && (actual.is_a?(PArrayType))
+    def describe_tuple(expected, original, actual, path, size_mismatch_class)
+      return EMPTY_ARRAY if expected == actual || expected.types.empty? && (actual.is_a?(PArrayType))
       expected_size = expected.size_type || TypeFactory.range(*expected.size_range)
 
       if actual.is_a?(PTupleType)
@@ -852,10 +876,11 @@ module Types
         if expected_size.assignable?(actual_size)
           etypes = expected.types
           descriptions = []
-          actual.types.each_with_index do |atype, index|
-            adx = index >= etypes.size ? etypes.size - 1 : index
-            etype = etypes[adx]
-            descriptions.concat(describe(etypes[adx], atype, path + [ArrayPathElement.new(adx)]))
+          unless etypes.empty?
+            actual.types.each_with_index do |atype, index|
+              adx = index >= etypes.size ? etypes.size - 1 : index
+              descriptions.concat(describe(etypes[adx], atype, path + [ArrayPathElement.new(adx)]))
+            end
           end
           descriptions
         else
@@ -868,7 +893,7 @@ module Types
           # Array of anything can not be assigned (unless tuple is tuple of anything) - this case
           # was handled at the top of this method.
           #
-          [TypeMismatch.new(path, expected, actual)]
+          [TypeMismatch.new(path, original, actual)]
         else
           expected_size = expected.size_type || TypeFactory.range(*expected.size_range)
           actual_size = actual.size_type || PCollectionType::DEFAULT_SIZE
@@ -883,40 +908,46 @@ module Types
           end
         end
       else
-        [TypeMismatch.new(path, expected, actual)]
+        [TypeMismatch.new(path, original, actual)]
       end
     end
 
-    def describe_PCallableType(expected, actual, path)
+    def describe_PCallableType(expected, original, actual, path)
       if actual.is_a?(PCallableType)
         # nil param_types means, any other Callable is assignable
-        if expected.param_types.nil?
+        if expected.param_types.nil? && expected.return_type.nil?
           EMPTY_ARRAY
         else
           # NOTE: these tests are made in reverse as it is calling the callable that is constrained
           # (it's lower bound), not its upper bound
           param_errors = describe_argument_tuple(expected.param_types, actual.param_types, path)
           if param_errors.empty?
-            # names are ignored, they are just information
-            # Blocks must be compatible
-            this_block_t = expected.block_type || PUndefType::DEFAULT
-            that_block_t = actual.block_type || PUndefType::DEFAULT
-            if that_block_t.assignable?(this_block_t)
-              EMPTY_ARRAY
+            this_return_t = expected.return_type || PAnyType::DEFAULT
+            that_return_t = actual.return_type || PAnyType::DEFAULT
+            unless this_return_t.assignable?(that_return_t)
+              [TypeMismatch.new(path + [ReturnTypeElement.new], this_return_t, that_return_t)]
             else
-              [TypeMismatch.new(path + BlockPathElement.new, this_block_t, that_block_t)]
+              # names are ignored, they are just information
+              # Blocks must be compatible
+              this_block_t = expected.block_type || PUndefType::DEFAULT
+              that_block_t = actual.block_type || PUndefType::DEFAULT
+              if that_block_t.assignable?(this_block_t)
+                EMPTY_ARRAY
+              else
+                [TypeMismatch.new(path + [BlockPathElement.new], this_block_t, that_block_t)]
+              end
             end
           else
             param_errors
           end
         end
       else
-        [TypeMismatch.new(path, expected, actual)]
+        [TypeMismatch.new(path, original, actual)]
       end
     end
 
-    def describe_PAnyType(expected, actual, path)
-      expected.assignable?(actual) ? EMPTY_ARRAY : [TypeMismatch.new(path, expected, actual)]
+    def describe_PAnyType(expected, original, actual, path)
+      expected.assignable?(actual) ? EMPTY_ARRAY : [TypeMismatch.new(path, original, actual)]
     end
 
     class UnresolvedTypeFinder
@@ -942,33 +973,38 @@ module Types
       if unresolved
         [UnresolvedTypeReference.new(path, unresolved)]
       else
-        case expected
-        when PVariantType
-          describe_PVariantType(expected, actual, path)
-        when PStructType
-          describe_PStructType(expected, actual, path)
-        when PHashType
-          describe_PHashType(expected, actual, path)
-        when PTupleType
-          describe_PTupleType(expected, actual, path)
-        when PArrayType
-          describe_PArrayType(expected, actual, path)
-        when PCallableType
-          describe_PCallableType(expected, actual, path)
-        when POptionalType
-          describe_POptionalType(expected, actual, path)
-        when PPatternType
-          describe_PPatternType(expected, actual, path)
-        when PEnumType
-          describe_PEnumType(expected, actual, path)
-        when PTypeAliasType
-          describe_PTypeAliasType(expected, actual, path)
-        else
-          describe_PAnyType(expected, actual, path)
-        end
+        internal_describe(expected.normalize, expected, actual, path)
       end
     end
 
+    def internal_describe(expected, original, actual, path)
+      case expected
+      when PVariantType
+        describe_PVariantType(expected, original, actual, path)
+      when PStructType
+        describe_PStructType(expected, original, actual, path)
+      when PHashType
+        describe_PHashType(expected, original, actual, path)
+      when PTupleType
+        describe_PTupleType(expected, original, actual, path)
+      when PArrayType
+        describe_PArrayType(expected, original, actual, path)
+      when PCallableType
+        describe_PCallableType(expected, original, actual, path)
+      when POptionalType
+        describe_POptionalType(expected, original, actual, path)
+      when PPatternType
+        describe_PPatternType(expected, original, actual, path)
+      when PEnumType
+        describe_PEnumType(expected, original, actual, path)
+      when PTypeAliasType
+        describe_PTypeAliasType(expected, original, actual, path)
+      else
+        describe_PAnyType(expected, original, actual, path)
+      end
+    end
+    private :internal_describe
+
     # Produces a string for the signature(s)
     #
     # @api private
