@@ -112,7 +112,7 @@ module Puppet
         begin
           provider.update
         rescue => detail
-          self.fail Puppet::Error, "Could not update: #{detail}", detail
+          self.fail Puppet::Error, _("Could not update: %{detail}") % { detail: detail }, detail
         end
 
         if current == :absent
@@ -126,7 +126,7 @@ module Puppet
         begin
           provider.install
         rescue => detail
-          self.fail Puppet::Error, "Could not update: #{detail}", detail
+          self.fail Puppet::Error, _("Could not update: %{detail}") % { detail: detail }, detail
         end
 
         if self.retrieve == :absent
@@ -161,7 +161,7 @@ module Puppet
                 @latest = provider.latest
                 @lateststamp = Time.now.to_i
               rescue => detail
-                error = Puppet::Error.new("Could not get latest version: #{detail}")
+                error = Puppet::Error.new(_("Could not get latest version: %{detail}") % { detail: detail })
                 error.set_backtrace(detail.backtrace)
                 raise error
               end
@@ -207,7 +207,7 @@ module Puppet
       # Provide a bit more information when logging upgrades.
       def should_to_s(newvalue = @should)
         if @latest
-          @latest.to_s
+          super(@latest)
         else
           super(newvalue)
         end
@@ -261,7 +261,7 @@ module Puppet
 
       validate do |value|
         if !value.is_a?(String)
-          raise ArgumentError, "Name must be a String not #{value.class}"
+          raise ArgumentError, _("Name must be a String not %{klass}") % { klass: value.class }
         end
       end
     end
